local T, check_number, rowhighlight = ...

local number_ranges = {
	{low = 0, high = 255,},
	{low = 0.01, high = 2, dp = 2},
	{low = 0, high = gkinterface.GetXResolution()},
	{low = 0, high = gkinterface.GetYResolution()},
}

local function get_control_width(generator, str, font)
	local c = iup[generator]{[generator == "text" and "value" or "title"] = str, font = font, expand = "NO"}
	iup.Map(c)
	local w = tonumber(c.w)
	iup.Destroy(c)
	return w
end

local function create_subdlg(ctrl)
	local dlg = iup.dialog{ctrl, border="NO", menubox="NO", resize="NO", bgcolor="0 0 0 0 *",}
	dlg.visible = "YES"
	return dlg
end

local function create_scrollbox(content)
	local scrollbox = iup.stationsublist{{}, control = "YES", expand = "YES", size = "%52x%60"}
	iup.Append(scrollbox, create_subdlg(content))
	scrollbox[1] = 1
	return scrollbox
end
local function get_control_dimensions(size)
	local c = iup.canvas{size = size, expand = "NO"}
	iup.Map(c)
	local w, h = tonumber(c.w), tonumber(c.h)
	iup.Destroy(c)
	return w, h
end

local function check_decimal_prefix(str, dp)
	local lead = str:match("^0?%.?$")
	if (lead == "") then lead = nil end
	local trail = str:match("^0?%.(0-)$")
	return trail and trail:len() < dp or lead
end


-- ### ######################################################### ###
-- ###                                                           ###
-- ###                       Positioner UI                       ###
-- ###                                                           ###
-- ### ######################################################### ###
function BuildPositionerUI(horizontal)

	local dlg
	local container
	local position
	local savebtn, cancelbtn, resetbtn

	local xRes = gkinterface.GetXResolution() - T.UI.display_container.w
	local yRes = gkinterface.GetYResolution() - T.UI.display_container.h
	local w, h = get_control_dimensions("THIRDxTHIRD")

-- ### ######################################################### ###
-- ### Positioner UI - Interface Layout                          ###
-- ### ######################################################### ###

	position = iup.canvas{
		size = w..'x'..w,
		scrollbar = horizontal and "HORIZONTAL" or "YES",
		xmin = 0,
		xmax = xRes,
		dx = Font.Default / ((w / xRes)-.05),
		ymin = 0,
		ymax = yRes,
		dy = Font.Default / ((w / yRes)-.05),
		border = "NO",
		expand = "NO",
	}

	w, h = get_control_width("stationbutton", string.rep("X", string.len("Cancel")))
	savebtn   = iup.stationbutton{title = "Save", size = w}
	cancelbtn = iup.stationbutton{title = "Cancel", size = w}
	resetbtn = iup.stationbutton{title = "Reset", size = w}

	container = iup.zbox{
		position,
		iup.vbox{
			savebtn,
			cancelbtn,
			resetbtn,
			alignment = "ACENTER",
			gap = w / 3,
		},
		all = "YES",
		alignment = "ACENTER",
	}

	dlg = iup.dialog{
		iup.vbox{
			iup.fill{},
			iup.hbox{
				iup.fill{},
				container,
				iup.fill{},
			},
			iup.fill{},
		},
		topmost = "YES",
		fullscreen = "YES",
		defaultesc = cancelbtn,
		bgcolor = "0 0 0 32 *"
	}


-- ### ######################################################### ###
-- ### Positioner UI - Callbacks                                 ###
-- ### ######################################################### ###
	local orig_width
	local orig_x, orig_y

	function position:scroll_cb(op, posx, posy)
		if (horizontal) then
			T.UI.display:Resize(posx + T.name_width_min, posy)
		else
			T.UI.display_container:PositionDisplay(posx, posy)
		end
		--if (op) then make_dirty() end
	end

	function savebtn:action()
		if (horizontal) then
			T.optionsdlg:UpdateWidth(position.posx + T.name_width_min) -- Not going to rebuild
		else
			T.optionsdlg:UpdatePosition(position.posx, position.posy)
		end
		HideDialog(iup.GetDialog(self))
		ShowDialog(T.optionsdlg)
	end

	function cancelbtn:action()
		if (horizontal) then
			--T:RebuildDisplay() -- have to rebuild since the tabs and filters were removed
			position.posx = orig_width
			position:scroll_cb(nil, orig_width)
		else
			T.UI.display_container:PositionDisplay(T.x, T.y) -- return the targetlist to the original position if canceled
		end
		HideDialog(iup.GetDialog(self))
		ShowDialog(T.optionsdlg)
	end

	function resetbtn:action()
		if (horizontal) then
-- 			position.posx = T.name_width
			position.posx = orig_width
			position:scroll_cb(nil, orig_width)
		else
			position.posx, position.posy = T.UI.display_container:PositionDisplay(T.x, T.y)
		end
	end

	function dlg:Show(posx, posy)
		if (horizontal) then
			T.UI.display_container[1][3].visible = "NO" -- filters
			T.UI.display_container[1][1].visible = "NO" -- tabs
			orig_width = T.UI.display:GetSize() - T.name_width_min
			position.posx = orig_width
		else
			orig_x, orig_y = tonumber(T.UI.display_container.cx), tonumber(T.UI.display_container.cy)
			position.posx, position.posy = T.UI.display_container:PositionDisplay(posx, posy) -- rebuild won't be necessary later if canceled
		end
		ShowDialog(self)
	end

	function dlg:show_cb()
	end

	function dlg:close_cb()
		iup.Destroy(self)
	end

	iup.Map(dlg)
	T.posdlg = dlg
	return dlg
end

-- ### ######################################################### ###
-- ###                                                           ###
-- ###                       Options UI                          ###
-- ###                                                           ###
-- ### ######################################################### ###

function T:BuildOptionsUI()

	local dlg
	local options_container, left_side_container, right_side_container
	local fontscale, fontscaledisplay
	local highlight_R, highlight_G, highlight_B, highlight_A, highlight_example, highlight, highlight_refresh
	local num_rows
	local name_width
	local xpos, ypos, setposbtn, setwidthbtn
	local hide_friendly, hide_group, hide_turrets
	local show_npc_ship_types, show_npc_faction, show_all_players, show_hive_only, auto_pin_players
	local pin_sort, pin_sort_bg
	local cancelbtn, applybtn, okbtn, resetbtn

	local column_gap, row_gap, alpha, drawcenter, compatibility_highlight, compatibility_tab


-- ### ######################################################### ###
-- ### Options UI - Interface Layout                             ###
-- ### ######################################################### ###
	local text_width  = get_control_width("text", "00000")
	local label_width = get_control_width("label", "Sort pinned targets by")

	fontscale             = iup.canvas{size = string.format("%dx%d", 200, Font.Default), scrollbar = "HORIZONTAL", xmin = 1, xmax = number_ranges[2].high * 100, posx = 100, dx = Font.Default / ((200 / 200) - 0.05), border = "NO"}
	fontscaledisplay        = iup.text{size = text_width}
	highlight_R             = iup.text{size = text_width}
	highlight_G             = iup.text{size = text_width}
	highlight_B             = iup.text{size = text_width}
	highlight_A             = iup.text{size = text_width}
	num_rows                = iup.text{size = text_width}
	name_width              = iup.text{size = text_width}
	xpos                    = iup.text{size = text_width}
	ypos                    = iup.text{size = text_width}
	column_gap              = iup.text{size = text_width}
	row_gap                 = iup.text{size = text_width}
	alpha                   = iup.text{size = text_width}
	drawcenter              = iup.stationtoggle{title = "Draw Center of Display",}
	compatibility_highlight = iup.stationtoggle{title = "Row Highlight",}
	compatibility_tab       = iup.stationtoggle{title = "Tab Highlight",}
	hide_friendly           = iup.stationtoggle{title = "Hide Friendly Targets",}
	hide_group              = iup.stationtoggle{title = "Hide Group Members",}
	hide_turrets            = iup.stationtoggle{title = "Hide Turrets",}
	show_hive_only          = iup.stationtoggle{title = "Show Only Hive",}
	show_npc_ship_types     = iup.stationtoggle{title = "Show NPC Ship Type",}
	show_npc_faction        = iup.stationtoggle{title = "Show NPC Faction",}
	show_pc_ship_types      = iup.stationtoggle{title = "Show PC Ship Type",}
	show_all_players        = iup.stationtoggle{title = "Show All Players",}
	auto_pin_players        = iup.stationtoggle{title = "Auto-pin Players",}
	pin_sort                = iup.stationhighopacitysublist{"(None)", "Name", "Distance", dropdown = "YES", border = "YES"}
	iup.Map(pin_sort)
	pin_sort_bg           = iup.text{size = pin_sort.w - Font.Default, active = "NO"}

	highlight = iup.label{title = "Highlight Example", expand = "NO", fgcolor = "255 255 255 255 *"}
	highlight_refresh = iup.canvas{expand = "YES", border = "NO"}
	highlight_example = rowhighlight{highlight, bgcolor = "0 0 0 0 *"}

-- 		iup.zbox{
-- 			highlight_refresh,
-- 			iup.label{title = "Highlight Example", expand = "NO"},
-- 			all = "YES",
-- 			expand = "NO",
-- 		}
-- 	}

	setposbtn        = iup.stationbutton{title = "Positioner"}
	setwidthbtn      = iup.stationbutton{title = "Width"}
	resetbtn         = iup.stationbutton{title = "Reset"}
	cancelbtn        = iup.stationbutton{title = "Cancel"}
	applybtn         = iup.stationbutton{title = "Apply", active = "NO"}
	okbtn            = iup.stationbutton{title = "OK"}

	local gap = 20
	local content_gap = 10

	left_side_container = create_scrollbox(iup.vbox{
		iup.label{title = "Appearance & Position"},
		iup.hbox{ -- fontscale
			iup.label{title = "Font Scale", size = label_width},
			iup.hbox{
				fontscale,
				fontscaledisplay,
				iup.fill{},
				gap = content_gap
			},
		},
		iup.hbox{ -- highlight
			iup.label{title = "Highlight", size = label_width},
			iup.hbox{
				iup.hbox{
					highlight_R,
					iup.label{title = "R "},
				},
				iup.hbox{
					highlight_G,
					iup.label{title = "G "},
				},
				iup.hbox{
					highlight_B,
					iup.label{title = "B "},
				},
				iup.hbox{
					highlight_A,
					iup.label{title = "A "},
				},
				iup.fill{},
				gap = content_gap,
			},
		},
		iup.hbox{ -- number of rows
			iup.label{title = "Number of Rows", size = label_width},
			iup.hbox{
				num_rows,
				iup.fill{},
				highlight_example,
			},
		},
		iup.hbox{ -- name width
			iup.label{title = "Name Width*", size = label_width},
			iup.hbox{
				name_width,
				iup.fill{},
				iup.label{title = "* blank is default"},
				iup.fill{},
			},
			setwidthbtn,
		},
		iup.hbox{ -- position
			iup.label{title = "Position*", size = label_width,},
			iup.hbox{
				iup.hbox{
					xpos,
					iup.label{title = "X "},
				},
				iup.hbox{
					ypos,
					iup.label{title = "Y "},
				},
				iup.fill{},
				setposbtn,
				gap = content_gap,
			},
		},
		iup.hbox{
			iup.label{title = "Column Gap", size = label_width},
			iup.hbox{
				column_gap,
			},
		},
		iup.hbox{
			iup.label{title = "Row Gap", size = label_width},
			iup.hbox{
				row_gap,
			},
		},
		iup.hbox{
			iup.label{title = "Display Alpha", size = label_width},
			iup.hbox{
				alpha,
			},
		},
		iup.hbox{
			iup.label{title = "", size = label_width},
			drawcenter,
		},
		iup.label{title = "Compatibility Mode", size = label_width},
		iup.hbox{
			iup.label{title = "", size = label_width},
			compatibility_highlight,
		},
		iup.hbox{
			iup.label{title = "", size = label_width},
			compatibility_tab,
		},
		gap = content_gap,
		margin = "2x2",
	})

	right_side_container = iup.vbox{
		iup.label{title = "Display Options"},
		iup.hbox{
			iup.label{title = "Pinned Sorting:", size = label_width},
			iup.zbox{
				pin_sort_bg,
				pin_sort,
				all = "YES",
				alignment = "NW",
			},
			gap = content_gap,
		},
		auto_pin_players,
		hide_friendly,
		hide_group,
		hide_turrets,
		show_hive_only,
		show_npc_ship_types,
		show_npc_faction,
		show_pc_ship_types,
		show_all_players,
		iup.fill{},
		gap = content_gap,
		margin = "2x2",

	}

	options_container = iup.hbox{
		iup.vbox{
			iup.stationhighopacityframebg{
				iup.pdarootframe{
					iup.hbox{
						iup.label{title = string.format("%s %s Options", targetlist.config_name, targetlist.version)},
						iup.fill{expand = "YES"},
					},
				},
			},
			iup.stationhighopacityframebg{
				iup.pdarootframe{
					iup.vbox{
						iup.hbox{
							iup.pdarootframe{left_side_container,},
							iup.pdarootframe{right_side_container,},
							gap = gap,
						},
						-- ------------------
						gap = gap,
						expand = "NO",
						margin = "5x5",
					},
				},
			},
			iup.stationhighopacityframebg{
				iup.pdarootframe{
					iup.hbox{
						resetbtn,
						iup.fill{},
						cancelbtn,
						applybtn,
						okbtn,
						gap = gap,
					},
				},
			},
			gap = gap,
		},
		expand = "NO",
	}

	dlg = iup.dialog{
		iup.vbox{
			iup.fill{},
			iup.hbox{
				iup.fill{},
				options_container,
				iup.fill{},
			},
			iup.fill{},
		},
		topmost = "YES",
		fullscreen = "YES",
		defaultesc = cancelbtn,
		bgcolor = "0 0 0 64 *"
	}


-- ### ######################################################### ###
-- ### Options UI - Callbacks                                    ###
-- ### ######################################################### ###

	local apply_settings, load_settings, reset_settings, merge_color_parts, make_dirty, is_mobile_input_good
	local dirty_width -- if the widthener was used to adjust the width, then the display needs to be rebuilt, even on cancel

	local function num_text_action(Lval, Uval, dp, action)
		-- Action function that will only allow number-based entries within a given range
		-- NOTE:  Doesn't support negative values
		-- <Lval>, <Uval>:  Lower and Upper bounds in the number range
		-- [dp]: how many decimal places to consider, or the [action] function
		-- [action]:  function to be called upon success
		if (type(dp) == "function") then
			action = dp
			dp = nil
		end

		return function(self, c, after)
			local num = tonumber(after)
			if (c == 8  or c == iup.K_BS or -- backspace
				c == 9  or -- tab
				c == 27 or -- escape
				c == 13 or -- enter
				dp and (c == 46 or c == 48) and check_decimal_prefix(after, dp) or -- 46: .   48: 0
				check_number(num, Lval, Uval, dp)
			) then
				if (not dp and c == 46) then return iup.IGNORE end -- if there's no decimal allowed and they try to enter a decimal point
				if (type(action) == "function") then action(c, num) end
				make_dirty(is_mobile_input_good)
				return iup.DEFAULT
			else
				return iup.IGNORE
			end
		end
	end

	function fontscale:scroll_cb(op, posx, posy)
		fontscaledisplay.value = string.format("%4.2f", posx / 100)
		if (op) then make_dirty() end -- only mark dialog as dirty if the slider is dragged
	end

	fontscaledisplay.action = num_text_action(number_ranges[2].low, number_ranges[2].high, number_ranges[2].dp, function(c, after)
		after = (after or 0) < 0.01 and 0.01 or after
		fontscale.posx = after * 100
		self.value = 0.01
	end)

	highlight_R.action = num_text_action(number_ranges[1].low, number_ranges[1].high, function(c, after)
		highlight_example.bgcolor = merge_color_parts(
			after ~= "" and after or T.defaults.target_color.val:match("(%d+) %d+ %d+ %d+"),
			highlight_G.value,
			highlight_B.value,
			highlight_A.value,
			"*"
		)
	end)
	highlight_G.action = num_text_action(number_ranges[1].low, number_ranges[1].high, function(c, after)
		highlight_example.bgcolor = merge_color_parts(
			highlight_R.value,
			after ~= "" and after or T.defaults.target_color.val:match("%d+ (%d+) %d+ %d+"),
			highlight_B.value,
			highlight_A.value,
			"*"
		)
	end)
	highlight_B.action = num_text_action(number_ranges[1].low, number_ranges[1].high, function(c, after)
		highlight_example.bgcolor = merge_color_parts(
			highlight_R.value,
			highlight_G.value,
			after ~= "" and after or T.defaults.target_color.val:match("%d+ %d+ (%d+) %d+"),
			highlight_A.value,
			"*"
		)
	end)
	highlight_A.action = num_text_action(number_ranges[1].low, number_ranges[1].high, function(c, after)
		highlight_example.bgcolor = merge_color_parts(
			highlight_R.value,
			highlight_G.value,
			highlight_B.value,
			after ~= "" and after or T.defaults.target_color.val:match("%d+ %d+ %d+ (%d+)"),
			"*"
		)
	end)

	function highlight_refresh:action(x, y)
		if ((check_number(highlight_R.value, number_ranges[1].low, number_ranges[1].high, number_ranges[1].dp) or highlight_R.value == "") and
			(check_number(highlight_G.value, number_ranges[1].low, number_ranges[1].high, number_ranges[1].dp) or highlight_R.value == "") and
			(check_number(highlight_B.value, number_ranges[1].low, number_ranges[1].high, number_ranges[1].dp) or highlight_R.value == "") and
			(check_number(highlight_A.value, number_ranges[1].low, number_ranges[1].high, number_ranges[1].dp) or highlight_R.value == "")
		) then
			highlight_example.bgcolor = merge_color_parts(
				highlight_R.value or T.defaults.target_color.val:match("(%d+) %d+ %d+ %d+"),
				highlight_G.value or T.defaults.target_color.val:match("%d+ (%d+) %d+ %d+"),
				highlight_B.value or T.defaults.target_color.val:match("%d+ %d+ (%d+) %d+"),
				highlight_A.value or T.defaults.target_color.val:match("%d+ %d+ %d+ (%d+)"),
				"*"
			)
		end
	end

	num_rows.action = num_text_action(1)
	name_width.action = num_text_action(1)
	column_gap.action = num_text_action(0)
	row_gap.action = num_text_action(0)
	alpha.action = num_text_action(number_ranges[1].low, number_ranges[1].high, number_ranges[1].dp)

	local function pos_verify_func(c, after)
	end
	xpos.action = num_text_action(number_ranges[3].low, number_ranges[3].high, pos_verify_func)
	ypos.action = num_text_action(number_ranges[4].low, number_ranges[4].high, pos_verify_func)

	function pin_sort:action(t, i, v) if (v == 1) then make_dirty() end end

	function auto_pin_players:action(v) make_dirty() end
	function hide_friendly:action(v) make_dirty() end
	function hide_group:action(v) make_dirty() end
	function hide_turrets:action(v) make_dirty() end
	function show_hive_only:action(v) make_dirty() end
	function show_npc_ship_types:action(v) make_dirty() end
	function show_npc_faction:action(v) make_dirty() end
	function show_pc_ship_types:action(v) make_dirty() end
	function show_all_players:action(v) make_dirty() end
	function drawcenter:action(v) make_dirty() end
	function compatibility_highlight:action(v) make_dirty() end
	function compatibility_tab:action(v) make_dirty() end

	function okbtn:action()
		if (applybtn.active == "YES") then
			if (not applybtn:action()) then return end
		end
		T:RebuildDisplay()
		--iup.Close(dlg)
		HideDialog(iup.GetDialog(self))
	end

	function applybtn:action()
		return apply_settings()
	end

	function cancelbtn:action()
		T.UI.display_container:PositionDisplay(T.x, T.y) -- return the display to it's original position in case it was moved by the positioner.
		if (dirty_width) then
			T:RebuildDisplay()
		else
			T.UI.display_container:PositionDisplay(T.x, T.y) -- return the display to it's original position in case it was moved by the positioner.
		end
		HideDialog(iup.GetDialog(self))
	end

	function resetbtn:action()
		reset_settings()
		make_dirty()
	end

	function setposbtn:action()
		dlg.show_positioner = true
		--BuildPositionerUI():Show(xpos.value, ypos.value)
		BuildPositionerUI()
		T.posdlg:Show(xpos.value, ypos.value)
		HideDialog(dlg) --iup.GetDialog(self))
	end

	function setwidthbtn:action()
		dirty_width = true
		dlg.show_positioner = true
		--BuildPositionerUI():Show(xpos.value, ypos.value)
		BuildPositionerUI(true)
		HideDialog(dlg) --iup.GetDialog(self))
		T.posdlg:Show(xpos.value, ypos.value)
	end

	function dlg:UpdatePosition(posx, posy, init)
		xpos.value = posx
		ypos.value = posy
		if (not init) then make_dirty() end
	end

	function dlg:UpdateWidth(w, posy, init)
		name_width.value = w
		if (not init) then make_dirty() end
	end

	function dlg:show_cb()
		-- If returning from the positioner, don't reload the options.
		if (self.show_positioner) then
			self.show_positioner = false -- for some reason assigning nil didn't clear the variable
		else
			load_settings()
		end

	end

	function dlg:close_cb()
		if (self.show_positioner) then
		else
		end
	end

-- --------------------------------------------------
	local function set_text_value(ctrl, val)
		ctrl.value = val or ""
		ctrl.fgcolor = "255 255 255"
	end

	load_settings = function()
		set_text_value(fontscaledisplay, T.fontscale)
		fontscale.posx = fontscaledisplay.value * 100
		local r, g, b, a = T.current_target_color:match("(%d+) (%d+) (%d+) (%d+)")
		set_text_value(highlight_R,     r)
		set_text_value(highlight_G,     g)
		set_text_value(highlight_B,     b)
		set_text_value(highlight_A,     a)
		highlight_example.bgcolor     = T.current_target_color -- highlight sample
		set_text_value(num_rows,        T.list_count)
		set_text_value(name_width,      T.name_width)
		set_text_value(column_gap,      T.display_column_gap)
		set_text_value(row_gap,         T.display_row_gap)
		set_text_value(alpha,           T.display_alpha)
		drawcenter.value              = T.display_drawcenter    and "ON" or "OFF"
		compatibility_highlight.value = T.compatibility_mode_h  and "ON" or "OFF"
		compatibility_tab.value       = T.compatibility_mode_t  and "ON" or "OFF"
		pin_sort.value                = T.pin_sort
		auto_pin_players.value        = T.auto_pin_players      and "ON" or "OFF"
		hide_friendly.value           = T.hide_friendly         and "ON" or "OFF"
		hide_group.value              = T.hide_group            and "ON" or "OFF"
		hide_turrets.value            = T.hide_turrets          and "ON" or "OFF"
		show_hive_only.value          = T.show_hive_only        and "ON" or "OFF"
		show_npc_ship_types.value     = T.show_npc_ship_types   and "ON" or "OFF"
		show_npc_faction.value        = T.show_npc_faction      and "ON" or "OFF"
		show_pc_ship_types.value      = T.show_pc_ship_types    and "ON" or "OFF"
		show_all_players.value        = T.show_all_players      and "ON" or "OFF"
		dlg:UpdatePosition(T.x or T.UI.display_container.cx, T.y or T.UI.display_container.cy, true)
	end

	reset_settings = function()
		set_text_value(fontscaledisplay, T.defaults.fontscale.val)
		fontscale.posx = fontscaledisplay.value * 100
		local r, g, b, a = T.defaults.target_color.val:match("(%d+) (%d+) (%d+) (%d+)")
		highlight.bgcolor = T.defaults.target_color.val
		set_text_value(highlight_R, r)
		set_text_value(highlight_G, g)
		set_text_value(highlight_B, b)
		set_text_value(highlight_A, a)
		set_text_value(num_rows, T.defaults.list_count.val)
		set_text_value(name_width, "")
		set_text_value(xpos, "")
		set_text_value(ypos, "")
		set_text_value(column_gap, T.defaults.display_column_gap.val)
		set_text_value(row_gap, T.defaults.display_row_gap.val)
		set_text_value(alpha, T.defaults.display_alpha.val)
		drawcenter.value              = (T.defaults.display_drawcenter.val == 1)    and "ON" or "OFF"
		compatibility_highlight.value = (T.defaults.compatibility_mode_h.val == 1)  and "ON" or "OFF"
		compatibility_tab.value       = (T.defaults.compatibility_mode_t.val == 1)  and "ON" or "OFF"
		pin_sort.value                = (T.defaults.pin_sort.val)
		auto_pin_players.value        = (T.defaults.auto_pin_players.val == 1)      and "ON" or "OFF"
		hide_friendly.value           = (T.defaults.hide_friendly.val == 1)         and "ON" or "OFF"
		hide_group.value              = (T.defaults.hide_group.val == 1)            and "ON" or "OFF"
		hide_turrets.value            = (T.defaults.hide_turrets.val == 1)          and "ON" or "OFF"
		show_hive_only.value          = (T.defaults.show_hive_only.val == 1)        and "ON" or "OFF"
		show_npc_ship_types.value     = (T.defaults.show_npc_ship_types.val == 1)   and "ON" or "OFF"
		show_npc_faction.value        = (T.defaults.show_npc_faction.val == 1)      and "ON" or "OFF"
		show_pc_ship_types.value      = (T.defaults.show_pc_ship_types.val == 1)    and "ON" or "OFF"
		show_all_players.value        = (T.defaults.show_all_players.val == 1)      and "ON" or "OFF"
		make_dirty()
	end

	apply_settings = function()
		if (not is_mobile_input_good()) then return false end

		T.fontscale            = check_number(fontscaledisplay.value, number_ranges[2].low, number_ranges[2].high, number_ranges[2].dp) or T.defaults.fontscale.val
		T.current_target_color = merge_color_parts(
		                             highlight_R.value or T.defaults.target_color.val:match("(%d+) %d+ %d+ %d+"),
		                             highlight_G.value or T.defaults.target_color.val:match("%d+ (%d+) %d+ %d+"),
		                             highlight_B.value or T.defaults.target_color.val:match("%d+ %d+ (%d+) %d+"),
		                             highlight_A.value or T.defaults.target_color.val:match("%d+ %d+ %d+ (%d+)"),
		                             "*"
		                         )
		T.list_count           = num_rows.value ~= "" and tonumber(num_rows.value) or T.defaults.list_count.val
		T.name_width           = name_width.value ~= "" and name_width.value ~= T.defaults.name_width.val and tonumber(name_width.value) or nil
		T.x                    = xpos.value ~= "" and xpos.value ~= T.defaults.x.val and tonumber(xpos.value) or nil
		T.y                    = ypos.value ~= "" and ypos.value ~= T.defaults.y.val  and tonumber(ypos.value) or nil
		T.display_column_gap   = column_gap.value ~= "" and tonumber(column_gap.value) or T.defaults.display_column_gap.val
		T.display_row_gap      = row_gap.value ~= "" and tonumber(row_gap.value) or T.defaults.display_row_gap.val
		T.display_alpha        = alpha.value ~= "" and tonumber(alpha.value) or T.defaults.display_alpha.val
		T.display_drawcenter   = drawcenter.value == "ON"
		T.compatibility_mode_h = compatibility_highlight.value == "ON"
		T.compatibility_mode_t = compatibility_tab.value == "ON"
		T.pin_sort             = tonumber(pin_sort.value)
		T.auto_pin_players     = auto_pin_players.value == "ON"
		T.hide_friendly        = hide_friendly.value == "ON"
		T.hide_group           = hide_group.value == "ON"
		T.hide_turrets         = hide_turrets.value == "ON"
		T.show_hive_only       = show_hive_only.value == "ON"
		T.show_npc_ship_types  = show_npc_ship_types.value == "ON"
		T.show_npc_faction     = show_npc_faction.value == "ON"
		T.show_pc_ship_types   = show_pc_ship_types.value == "ON"
		T.show_all_players     = show_all_players.value == "ON"
		T:SaveSettings()
		applybtn.active = "NO"
		return true
	end

	make_dirty = function(action)
		applybtn.active = "YES"
		if (type(action) == "function") then
			action()
		end
	end

	merge_color_parts = function(r, g, b, a, blend)
		r = r ~= "" and r or 0
		g = g ~= "" and g or 0
		b = b ~= "" and b or 0
		a = a ~= "" and a or 0
		return string.format("%s %s %s %s %s",r, g, b, a, blend)
	end

	local function mobile_check_control(ctrl, l, h, d)
		if (check_number(ctrl.value, l, h, d) or ctrl.value == "") then
			ctrl.fgcolor = "255 255 255"
		else
			ctrl.fgcolor = "255 0 0"
			return false
		end
		return true
	end

	is_mobile_input_good = function()
		local ok_to_apply = true
		ok_to_apply = mobile_check_control(fontscaledisplay, number_ranges[2].low, number_ranges[2].high, number_ranges[2].dp) and ok_to_apply
		ok_to_apply = mobile_check_control(highlight_R,      number_ranges[1].low, number_ranges[1].high, number_ranges[1].dp) and ok_to_apply
		ok_to_apply = mobile_check_control(highlight_B,      number_ranges[1].low, number_ranges[1].high, number_ranges[1].dp) and ok_to_apply
		ok_to_apply = mobile_check_control(highlight_G,      number_ranges[1].low, number_ranges[1].high, number_ranges[1].dp) and ok_to_apply
		ok_to_apply = mobile_check_control(highlight_A,      number_ranges[1].low, number_ranges[1].high, number_ranges[1].dp) and ok_to_apply
		ok_to_apply = mobile_check_control(num_rows,         1) and ok_to_apply
		ok_to_apply = mobile_check_control(name_width,       1) and ok_to_apply
		ok_to_apply = mobile_check_control(xpos,             0) and ok_to_apply
		ok_to_apply = mobile_check_control(ypos,             0) and ok_to_apply
		ok_to_apply = mobile_check_control(column_gap,       0) and ok_to_apply
		ok_to_apply = mobile_check_control(row_gap,          0) and ok_to_apply
		ok_to_apply = mobile_check_control(alpha,            number_ranges[1].low, number_ranges[1].high, number_ranges[1].dp) and ok_to_apply
		return ok_to_apply
	end


	iup.Map(dlg)
	self.optionsdlg = dlg
end

T:BuildOptionsUI()



